@@ -89,7 +89,7 @@ SV* crypt_otr_process_sending( CryptOTRUserState crypt_state, char* in_account,
 	int err;
 		
 	if( !who || !message )
-		return newSVpv( newmessage, 0);
+		return newSVpv( NULL, 0 );
 
 	err = otrl_message_sending( userstate, &otr_ops, crypt_state, 
 						   accountname, protocol, username, 
@@ -99,7 +99,7 @@ SV* crypt_otr_process_sending( CryptOTRUserState crypt_state, char* in_account,
 		/* Be *sure* not to send out plaintext */
 		//puts( "Oops, message not encrypted" );
 		char* ourm = strdup( "" );
-		free( message ); // This may cause bugs, I don't know how perl allocates memory, though it's probably with strdup
+		free( message );
 		message = ourm;
 	} else if ( newmessage ) {
 		/* Fragment the message if necessary, and send all but the last
@@ -114,8 +114,8 @@ SV* crypt_otr_process_sending( CryptOTRUserState crypt_state, char* in_account,
 									  newmessage, OTRL_FRAGMENT_SEND_ALL_BUT_LAST, &message);
 
 		// Checking for errors
-		if(err) {
-			crypt_otr_print_error("Fragmenting and sending message");
+		if (err) {
+            crypt_otr_print_error_code("fragmenting and sending message", err);
 		}
 		
 		otrl_message_free(newmessage);
@@ -125,52 +125,45 @@ SV* crypt_otr_process_sending( CryptOTRUserState crypt_state, char* in_account,
 }
 
 /*
- * returns whether a otr_message was received
- * sets *message to NULL, when it was an internal otr message
+ * returns plaintext if successful.
+ * returns status boolean in should_discard.
+ * if should_discard is true, this was an internal OTR protocol
+ *     message and should be ignored by the application.
  */
-SV*  crypt_otr_process_receiving( CryptOTRUserState crypt_state, char* in_accountname, char* in_protocol, int in_max, 
-						    char* who, char* sv_message )
+
+void crypt_otr_process_receiving( CryptOTRUserState crypt_state, const char* in_accountname,
+                                  const char* in_protocol, int in_max, const char* who,
+                                  const char* message, SV** out_plaintext, short *out_should_discard )
 {
-	char* message = strdup( sv_message  );
-	char* message_out = NULL;
-    	char* newmessage = NULL;
+    SV* ret;
+    char* ret_message = NULL;
 	OtrlTLV* tlvs = NULL;
 	OtrlTLV* tlv = NULL;
 	OtrlUserState userstate = crypt_state->otrl_state;
-	char* username = who;
-	int res;
-	const char* accountname = in_accountname;
-	const char* protocol = in_protocol;
 	ConnContext* context;
 	NextExpectedSMP nextMsg;
 
-	if( !who || !message )
-		return newSVpv( newmessage, 0);
+    *out_should_discard = 0;
 
-	res = otrl_message_receiving( userstate, &otr_ops, crypt_state, 
-							accountname, protocol, username, message,
-							&newmessage, &tlvs, NULL, NULL );
+	if( !who || !message ) {
+      *out_plaintext = newSVpvn(NULL, 0);
+      return;
+    }
 
-	if( newmessage ) {
-		char* ourm = malloc( strlen( newmessage ) + 1 );
-		if( ourm ) {
-			strcpy( ourm, newmessage );
-		}
-		otrl_message_free( newmessage );
-		free( message );
-		message = ourm;
-	}
+	*out_should_discard = otrl_message_receiving( userstate, &otr_ops, crypt_state, 
+                                              in_accountname, in_protocol, who, message,
+                                              &ret_message, &tlvs, NULL, NULL );
 
 	tlv = otrl_tlv_find( tlvs, OTRL_TLV_DISCONNECTED );
 	if( tlv ) {
 		/* Notify the user that the other side disconnected */
-		crypt_otr_handle_disconnection(crypt_state, username );
+		crypt_otr_handle_disconnection(crypt_state, who);
 	}
 
 	/* Keep track of our current progress in the Socialist Millionaires'
 	 * Protocol. */
-	context = otrl_context_find( userstate, username, 
-						    accountname, protocol, 0, NULL, NULL, NULL );
+	context = otrl_context_find( userstate, who, 
+						    in_accountname, in_protocol, 0, NULL, NULL, NULL );
 
 	if( context ) {
 		nextMsg = context->smstate->nextExpected;
@@ -241,16 +234,12 @@ SV*  crypt_otr_process_receiving( CryptOTRUserState crypt_state, char* in_accoun
 
 	otrl_tlv_free(tlvs);
 
-	/* If we're supposed to ignore this incoming message (because it's a
-	 * protocol message), set it to NULL, so that other plugins that
-	 * catch receiving-im-msg don't return 0, and cause it to be
-	 * displayed anyway. */
-	if (res) {
-		free(message);
-		message = NULL;
-	}
+    /* copy message */
+	ret = newSVpv(ret_message, 0);
+    *out_plaintext = ret;
 
-	return newSVpv( message, 0 );
+    /* we are responsible for freeing ret_message */
+    if (ret_message) otrl_message_free(ret_message);
 }
 
 /* Start the Socialist Millionaires' Protocol over the current connection,
@@ -322,71 +311,38 @@ unsigned short crypt_otr_get_pubkey_type( CryptOTRUserState in_state, char *acco
   return privkey->pubkey_type;
 }
 
-char* crypt_otr_get_privkey_fingerprint( CryptOTRUserState in_state, char *account, char *proto, int maxsize ) {
+SV* crypt_otr_get_privkey_fingerprint( CryptOTRUserState in_state, char *account, char *proto, int maxsize ) {
 	
-	char* fingerprint, fpr_ptr, accountname, protocol;
-	fingerprint = malloc(45);
-
-	// I don't know if it's safe practice to use the vars passed from XS
-	accountname = account;
-	protocol = proto;
+	char* fpr_ptr;
+	char fingerprint[45];
 	
-	printf( "About to call otrl_privkey_fingerprint\n");
-
-	fpr_ptr = otrl_privkey_fingerprint(in_state->otrl_state, fingerprint, accountname, protocol);
-	printf( "Done calling otrl_privkey_fingerprint\n");
+	printf("About to call otrl_privkey_fingerprint\n");
+	fpr_ptr = otrl_privkey_fingerprint(in_state->otrl_state, fingerprint, account, proto);
+	printf("Done calling otrl_privkey_fingerprint\n");
 	
-	if( fpr_ptr){
-		
-		printf( "About to create perl var\n");
-		// Create perl var, return it
-		SV *sig_sv = newSVpvn( fingerprint, 0);
-		
-		free(fingerprint);
-
-		printf("About to return\n");
-		return sig_sv;
-	} 
+    if (fpr_ptr) {
+      SV *sig_sv = newSVpvn(fingerprint, 0);
+      return sig_sv;
+	}
 	
 	crypt_otr_print_error("Getting fingerprint");
-	
-	// There was an error, free memory
-	free(fingerprint);
-	
-	return ;
+	return newSVpvn(NULL, 0);
 }
 
-char* crypt_otr_get_privkey_fingerprint_raw( CryptOTRUserState in_state, char *account, char *proto, int maxsize ) {
+SV* crypt_otr_get_privkey_fingerprint_raw( CryptOTRUserState in_state, char *account, char *proto, int maxsize ) {
+  char* fpr_ptr;
+  char hash[20];
 
-	char* fingerprint, fpr_ptr, accountname, protocol;
-	fingerprint = malloc(20);
+  fpr_ptr = otrl_privkey_fingerprint_raw(in_state->otrl_state, hash, account, proto);
 
-	// I don't know if it's safe practice to use the vars passed from XS
-	accountname = account;
-	protocol = proto;
+  if (fpr_ptr){
+    SV *sig_sv = newSVpvn(hash, 0);
+    return sig_sv;
+  } 
 
-	printf( "About to call otrl_privkey_fingerprint_raw\n");
+  crypt_otr_print_error("Getting fingerprint raw");
 
-	fpr_ptr = otrl_privkey_fingerprint_raw(in_state->otrl_state, fingerprint, accountname, protocol);
-
-	printf( "Done calling otrl_privkey_fingerprint_raw\n");
-
-
-	if( fpr_ptr){
-		// Create perl var, return it
-		SV *sig_sv = newSVpvn( fingerprint, 0);
-		
-		free(fingerprint);
-
-		return sig_sv;
-	} 
-
-	crypt_otr_print_error("Getting fingerprint raw");
-	
-	// There was an error, free memory
-	free(fingerprint);
-	
-	return ;
+  return newSVpvn(NULL, 0);
 }
 
 /* Read the fingerprint store from a file on disk into the 
@@ -426,11 +382,11 @@ SV* crypt_otr_sign( CryptOTRUserState in_state, char *account, char *proto, int
 
 	sign_error = otrl_privkey_sign(&sig, &siglen, privkey, msghash, strlen(msghash));
 
-	if(sign_error){
+	if (sign_error){
 		// There has to be a better way to pass an error,
 		// though string equality checking seems to be broken for the strings
 		// passed to Perl through XS, oh well
-		crypt_otr_print_error("Signing Data");
+        crypt_otr_print_error_code("Signing Data", sign_error);
 	} else {
 		// copy result, make SV
 		sig_sv = newSVpvn(sig, siglen);
@@ -473,8 +429,8 @@ unsigned int crypt_otr_verify( unsigned char *msghash, unsigned char *sig, unsig
 
 	gcry_error_t err = otrl_privkey_verify( sig, strlen(sig), pubkey_type, pubkey, msghash, strlen(msghash) );
 
-	if(err){
-		crypt_otr_print_error("Verifying data");
+	if (err){
+        crypt_otr_print_error_code("Verifying data", err);
 	}
 	
 	gcry_sexp_release(pubkey);
